@@ -1,31 +1,21 @@
 import os
-import json
-import logging
-from typing import List, Dict, Any
+import asyncio
+from typing import List, Any, Dict
 
-import httpx
 from dotenv import load_dotenv
 from mcp.server.fastmcp import FastMCP, Context
 
-# Import the ArmorWalletAPIClient, individual models, and the new container models.
+
+# Import base models
 from .armor_client import (
     ArmorWalletAPIClient,
-    WalletTokenPairs,
-    WalletTokenPairsContainer,         # New container for wallet/token pairs
+    calculate,
     WalletTokenBalance,
-    ConversionRequest,
-    ConversionRequestContainer,        # New container for conversion requests
     ConversionResponse,
-    SwapQuoteRequest,
-    SwapQuoteRequestContainer,           # New container for swap quote requests
     SwapQuoteResponse,
-    SwapTransactionRequest,
-    SwapTransactionRequestContainer,     # New container for swap transaction requests
     SwapTransactionResponse,
     Wallet,
-    TokenDetailsRequest,
-    TokenDetailsRequestContainer,        # New container for token details requests
-    TokenDetailsResponse,
+    TokenDetailsResponseContainer,
     GroupInfo,
     SingleGroupInfo,
     WalletInfo,
@@ -34,16 +24,42 @@ from .armor_client import (
     AddWalletToGroupResponse,
     GroupArchiveOrUnarchiveResponse,
     RemoveWalletFromGroupResponse,
-    UserWalletsAndGroupsResponse,
-    TransferTokensRequest,
-    TransferTokensRequestContainer,      # New container for transfer tokens requests
     TransferTokenResponse,
-    DCAOrderRequest,
-    DCAOrderRequestContainer,            # New container for DCA order requests
     DCAOrderResponse,
-    CancelDCAOrderRequest,
-    CancelDCAOrderRequestContainer,      # New container for cancel DCA order requests
-    CancelDCAOrderResponse
+    CancelDCAOrderResponse,
+    ListSingleGroupRequest,
+    TopTrendingTokensRequest,
+    CandleStickRequest,
+    StakeBalanceResponse,
+    ListWalletsRequest,
+    ListDCAOrderRequest,
+    ListOrderRequest,
+    PrivateKeyRequest,
+    WalletTokenPairsContainer,
+    ConversionRequestContainer,
+    SwapQuoteRequestContainer,
+    SwapTransactionRequestContainer,
+    TokenDetailsRequestContainer,
+    TransferTokensRequestContainer,
+    DCAOrderRequestContainer,
+    CancelDCAOrderRequestContainer,
+    CreateWalletRequestContainer,
+    ArchiveWalletsRequestContainer,
+    UnarchiveWalletRequestContainer,
+    CreateGroupsRequestContainer,
+    AddWalletToGroupRequestContainer,
+    ArchiveWalletGroupRequestContainer,
+    UnarchiveWalletGroupRequestContainer,
+    RemoveWalletsFromGroupRequestContainer,
+    CreateOrderRequestContainer,
+    CancelOrderRequestContainer,
+    CreateOrderResponseContainer,
+    CancelOrderResponseContainer,
+    StakeQuoteRequestContainer,
+    UnstakeQuoteRequestContainer,
+    StakeTransactionRequestContainer,
+    UnstakeTransactionRequestContainer,
+    RenameWalletRequestContainer,
 )
 
 # Load environment variables (e.g. BASE_API_URL, etc.)
@@ -62,9 +78,33 @@ armor_client = ArmorWalletAPIClient(ACCESS_TOKEN, base_api_url=BASE_API_URL) #,
 from armor_crypto_mcp import __version__
 @mcp.tool()
 async def get_armor_mcp_version():
-    # return  __version__
+    """Get the current Armor Wallet version"""
     return {'armor_version': __version__}
 
+@mcp.tool()
+async def wait_a_moment(seconds:float):
+    """Wait for some short amount of time, no more than 10 seconds"""
+    await asyncio.wait(seconds)
+    return {"waited": seconds}
+
+from datetime import datetime, timezone
+@mcp.tool()
+async def get_current_time():
+    """Gets the current time and date"""
+    return datetime.now(timezone.utc).strftime("%Y-%m-%d %H:%M:%S")
+
+@mcp.tool()
+async def calculator(expression:str, variables:dict[str, Any]):
+    """
+    Safely evaluates a mathematical or statistical expression string using Python syntax.
+
+    Supports arithmetic operations (+, -, *, /, **, %, //), list expressions, and a range of math and statistics functions: 
+    abs, round, min, max, len, sum, mean, median, stdev, variance, sin, cos, tan, sqrt, log, exp, floor, ceil, etc.
+
+    Custom variables can be passed via the 'variables' dict, including lists for time series data.
+    """
+    return calculate(expression, variables)
+
 @mcp.tool()
 async def get_wallet_token_balance(wallet_token_pairs: WalletTokenPairsContainer) -> List[WalletTokenBalance]:
     """
@@ -82,9 +122,9 @@ async def get_wallet_token_balance(wallet_token_pairs: WalletTokenPairsContainer
 
 
 @mcp.tool()
-async def conversion_api(conversion_requests: ConversionRequestContainer) -> List[ConversionResponse]:
+async def calculate_token_conversion(conversion_requests: ConversionRequestContainer) -> List[ConversionResponse]:
     """
-    Perform token conversion.
+    Perform token conversion quote between two tokens. Good for quickly calculating market prices.
     
     Expects a ConversionRequestContainer, returns a list of ConversionResponse.
     """
@@ -130,7 +170,7 @@ async def swap_transaction(swap_transaction_requests: SwapTransactionRequestCont
 
 
 @mcp.tool()
-async def get_all_wallets() -> List[Wallet]:
+async def get_all_wallets(get_all_wallets_requests: ListWalletsRequest) -> List[Wallet]:
     """
     Retrieve all wallets with balances.
     
@@ -139,14 +179,30 @@ async def get_all_wallets() -> List[Wallet]:
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[Wallet] = await armor_client.get_all_wallets()
+        result: List[Wallet] = await armor_client.get_all_wallets(get_all_wallets_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def get_all_orders(get_all_orders_requests: ListOrderRequest) -> List:
+    """
+    Retrieve all limit and stop loss orders.
+    
+    Returns a list of orders.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: List = await armor_client.list_orders(get_all_orders_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def get_token_details(token_details_requests: TokenDetailsRequestContainer) -> List[TokenDetailsResponse]:
+async def get_token_details(token_details_requests: TokenDetailsRequestContainer) -> TokenDetailsResponseContainer:
     """
     Retrieve token details.
     
@@ -155,7 +211,7 @@ async def get_token_details(token_details_requests: TokenDetailsRequestContainer
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[TokenDetailsResponse] = await armor_client.get_token_details(token_details_requests)
+        result: TokenDetailsResponseContainer = await armor_client.get_token_details(token_details_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
@@ -178,7 +234,7 @@ async def list_groups() -> List[GroupInfo]:
 
 
 @mcp.tool()
-async def list_single_group(group_name: str) -> SingleGroupInfo:
+async def list_single_group(list_single_group_requests: ListSingleGroupRequest) -> SingleGroupInfo:
     """
     Retrieve details for a single wallet group.
     
@@ -187,14 +243,14 @@ async def list_single_group(group_name: str) -> SingleGroupInfo:
     if not armor_client:
         return {"error": "Not logged in"}
     try:
-        result: SingleGroupInfo = await armor_client.list_single_group(group_name)
+        result: SingleGroupInfo = await armor_client.list_single_group(list_single_group_requests)
         return result
     except Exception as e:
         return {"error": str(e)}
 
 
 @mcp.tool()
-async def create_wallet(wallet_names_list: List[str]) -> List[WalletInfo]:
+async def create_wallet(create_wallet_requests: CreateWalletRequestContainer) -> List[WalletInfo]:
     """
     Create new wallets.
     
@@ -203,14 +259,14 @@ async def create_wallet(wallet_names_list: List[str]) -> List[WalletInfo]:
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[WalletInfo] = await armor_client.create_wallet(wallet_names_list)
+        result: List[WalletInfo] = await armor_client.create_wallet(create_wallet_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def archive_wallets(wallet_names_list: List[str]) -> List[WalletArchiveOrUnarchiveResponse]:
+async def archive_wallets(archive_wallet_requests: ArchiveWalletsRequestContainer) -> List[WalletArchiveOrUnarchiveResponse]:
     """
     Archive wallets.
     
@@ -219,14 +275,14 @@ async def archive_wallets(wallet_names_list: List[str]) -> List[WalletArchiveOrU
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[WalletArchiveOrUnarchiveResponse] = await armor_client.archive_wallets(wallet_names_list)
+        result: List[WalletArchiveOrUnarchiveResponse] = await armor_client.archive_wallets(archive_wallet_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def unarchive_wallets(wallet_names_list: List[str]) -> List[WalletArchiveOrUnarchiveResponse]:
+async def unarchive_wallets(unarchive_wallet_requests: UnarchiveWalletRequestContainer) -> List[WalletArchiveOrUnarchiveResponse]:
     """
     Unarchive wallets.
     
@@ -235,14 +291,14 @@ async def unarchive_wallets(wallet_names_list: List[str]) -> List[WalletArchiveO
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[WalletArchiveOrUnarchiveResponse] = await armor_client.unarchive_wallets(wallet_names_list)
+        result: List[WalletArchiveOrUnarchiveResponse] = await armor_client.unarchive_wallets(unarchive_wallet_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def create_groups(group_names_list: List[str]) -> List[CreateGroupResponse]:
+async def create_groups(create_groups_requests: CreateGroupsRequestContainer) -> List[CreateGroupResponse]:
     """
     Create new wallet groups.
     
@@ -251,14 +307,14 @@ async def create_groups(group_names_list: List[str]) -> List[CreateGroupResponse
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[CreateGroupResponse] = await armor_client.create_groups(group_names_list)
+        result: List[CreateGroupResponse] = await armor_client.create_groups(create_groups_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def add_wallets_to_group(group_name: str, wallet_names_list: List[str]) -> List[AddWalletToGroupResponse]:
+async def add_wallets_to_group(add_wallet_to_group_requests: AddWalletToGroupRequestContainer) -> List[AddWalletToGroupResponse]:
     """
     Add wallets to a specified group.
     
@@ -267,14 +323,14 @@ async def add_wallets_to_group(group_name: str, wallet_names_list: List[str]) ->
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[AddWalletToGroupResponse] = await armor_client.add_wallets_to_group(group_name, wallet_names_list)
+        result: List[AddWalletToGroupResponse] = await armor_client.add_wallets_to_group(add_wallet_to_group_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def archive_wallet_group(group_names_list: List[str]) -> List[GroupArchiveOrUnarchiveResponse]:
+async def archive_wallet_group(archive_wallet_group_requests: ArchiveWalletGroupRequestContainer) -> List[GroupArchiveOrUnarchiveResponse]:
     """
     Archive wallet groups.
     
@@ -283,14 +339,14 @@ async def archive_wallet_group(group_names_list: List[str]) -> List[GroupArchive
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[GroupArchiveOrUnarchiveResponse] = await armor_client.archive_wallet_group(group_names_list)
+        result: List[GroupArchiveOrUnarchiveResponse] = await armor_client.archive_wallet_group(archive_wallet_group_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def unarchive_wallet_group(group_names_list: List[str]) -> List[GroupArchiveOrUnarchiveResponse]:
+async def unarchive_wallet_group(unarchive_wallet_group_requests: UnarchiveWalletGroupRequestContainer) -> List[GroupArchiveOrUnarchiveResponse]:
     """
     Unarchive wallet groups.
     
@@ -299,14 +355,14 @@ async def unarchive_wallet_group(group_names_list: List[str]) -> List[GroupArchi
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[GroupArchiveOrUnarchiveResponse] = await armor_client.unarchive_wallet_group(group_names_list)
+        result: List[GroupArchiveOrUnarchiveResponse] = await armor_client.unarchive_wallet_group(unarchive_wallet_group_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
 
 
 @mcp.tool()
-async def remove_wallets_from_group(group_name: str, wallet_names_list: List[str]) -> List[RemoveWalletFromGroupResponse]:
+async def remove_wallets_from_group(remove_wallets_from_group_requests: RemoveWalletsFromGroupRequestContainer) -> List[RemoveWalletFromGroupResponse]:
     """
     Remove wallets from a specified group.
     
@@ -315,7 +371,7 @@ async def remove_wallets_from_group(group_name: str, wallet_names_list: List[str
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[RemoveWalletFromGroupResponse] = await armor_client.remove_wallets_from_group(group_name, wallet_names_list)
+        result: List[RemoveWalletFromGroupResponse] = await armor_client.remove_wallets_from_group(remove_wallets_from_group_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
@@ -331,9 +387,7 @@ async def transfer_tokens(transfer_tokens_requests: TransferTokensRequestContain
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        armor_client.logger.debug("Trying")
         result: List[TransferTokenResponse] = await armor_client.transfer_tokens(transfer_tokens_requests)
-        armor_client.logger.debug("We made it!")
         return result
     except Exception as e:
         return [{"error": str(e)}]
@@ -356,7 +410,7 @@ async def create_dca_order(dca_order_requests: DCAOrderRequestContainer) -> List
 
 
 @mcp.tool()
-async def list_dca_orders() -> List[DCAOrderResponse]:
+async def list_dca_orders(list_dca_order_requests: ListDCAOrderRequest) -> List[DCAOrderResponse]:
     """
     List all DCA orders.
     
@@ -365,7 +419,7 @@ async def list_dca_orders() -> List[DCAOrderResponse]:
     if not armor_client:
         return [{"error": "Not logged in"}]
     try:
-        result: List[DCAOrderResponse] = await armor_client.list_dca_orders()
+        result: List[DCAOrderResponse] = await armor_client.list_dca_orders(list_dca_order_requests)
         return result
     except Exception as e:
         return [{"error": str(e)}]
@@ -374,9 +428,9 @@ async def list_dca_orders() -> List[DCAOrderResponse]:
 @mcp.tool()
 async def cancel_dca_order(cancel_dca_order_requests: CancelDCAOrderRequestContainer) -> List[CancelDCAOrderResponse]:
     """
-    Cancel a DCA order.
-    
-    Expects a CancelDCAOrderRequestContainer, returns a list of CancelDCAOrderResponse.
+    Create a DCA order.
+
+    Note: Make a single or multiple dca_order_requests 
     """
     if not armor_client:
         return [{"error": "Not logged in"}]
@@ -385,6 +439,166 @@ async def cancel_dca_order(cancel_dca_order_requests: CancelDCAOrderRequestConta
         return result
     except Exception as e:
         return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def create_order(create_order_requests: CreateOrderRequestContainer) -> CreateOrderResponseContainer:
+    """
+    Create a order. Can be a limit or stop loss order
+    
+    Expects a CreateOrderRequestContainer, returns a CreateOrderResponseContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: CreateOrderResponseContainer = await armor_client.create_order(create_order_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def cancel_order(cancel_order_requests: CancelOrderRequestContainer) -> CancelOrderResponseContainer:
+    """
+    Cancel a limit or stop loss order.
+    
+    Expects a CancelOrderRequestContainer, returns a CancelOrderResponseContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: CancelOrderResponseContainer = await armor_client.cancel_order(cancel_order_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def stake_quote(stake_quote_requests: StakeQuoteRequestContainer) -> SwapQuoteRequestContainer:
+    """
+    Retrieve a stake quote.
+    
+    Expects a StakeQuoteRequestContainer, returns a SwapQuoteRequestContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: StakeQuoteRequestContainer = await armor_client.stake_quote(stake_quote_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+
+
+@mcp.tool()
+async def unstake_quote(unstake_quote_requests: UnstakeQuoteRequestContainer) -> SwapQuoteRequestContainer:
+    """
+    Retrieve an unstake quote.
+
+    Expects a UnstakeQuoteRequestContainer, returns a SwapQuoteRequestContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: UnstakeQuoteRequestContainer = await armor_client.unstake_quote(unstake_quote_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+
+
+@mcp.tool()
+async def stake_transaction(stake_transaction_requests: StakeTransactionRequestContainer) -> SwapTransactionRequestContainer:
+    """
+    Execute a stake transaction.
+    
+    Expects a StakeTransactionRequestContainer, returns a SwapTransactionRequestContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: SwapTransactionRequestContainer = await armor_client.stake_transaction(stake_transaction_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+
+
+@mcp.tool()
+async def unstake_transaction(unstake_transaction_requests: UnstakeTransactionRequestContainer) -> SwapTransactionRequestContainer:
+    """
+    Execute an unstake transaction.
+    
+    Expects a UnstakeTransactionRequestContainer, returns a SwapTransactionRequestContainer.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: SwapTransactionRequestContainer = await armor_client.unstake_transaction(unstake_transaction_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def get_top_trending_tokens(top_trending_tokens_requests: TopTrendingTokensRequest) -> List:
+    """
+    Get the top trending tokens in a particular time frame. Great for comparing market cap or volume.
+    
+    Expects a TopTrendingTokensRequest, returns a list of tokens with their details.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: List = await armor_client.top_trending_tokens(top_trending_tokens_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def get_stake_balances() -> StakeBalanceResponse:
+    """
+    Get the balance of staked SOL (jupSOL).
+    
+    Returns a StakeBalanceResponse.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: StakeBalanceResponse = await armor_client.get_stake_balances()
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def rename_wallets(rename_wallet_requests: RenameWalletRequestContainer) -> List:
+    """
+    Rename wallets.
+    
+    Expects a RenameWalletRequestContainer, returns a list.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: List = await armor_client.rename_wallet(rename_wallet_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+    
+
+@mcp.tool()
+async def get_token_candle_data(candle_stick_requests: CandleStickRequest) -> List:
+    """
+    Get the candle sticks.
+
+    Expects a CandleStickRequest, returns a list of candle sticks.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: List = await armor_client.get_market_candle_data(candle_stick_requests)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
 
 
 @mcp.prompt()
@@ -395,6 +609,20 @@ def login_prompt(email: str) -> str:
     return f"Please enter the Access token for your account {email}."
 
 
+@mcp.tool()
+async def send_key_to_telegram(private_key_request: PrivateKeyRequest) -> Dict:
+    """
+    Send the mnemonic or private key to telegram.
+    """
+    if not armor_client:
+        return [{"error": "Not logged in"}]
+    try:
+        result: Dict = await armor_client.send_key_to_telegram(private_key_request)
+        return result
+    except Exception as e:
+        return [{"error": str(e)}]
+
+
 def main():
     mcp.run()
     
